@@ -1,10 +1,13 @@
 # https://solidity.readthedocs.io/en/v0.4.24/units-and-global-variables.html
-from typing import List, Dict, Union
+from typing import List, Dict, Union, TYPE_CHECKING
 
 from slither.core.context.context import Context
 from slither.core.solidity_types import ElementaryType, TypeInformation
 from slither.exceptions import SlitherException
 
+if TYPE_CHECKING:
+    from slither.core.declarations import Import
+
 SOLIDITY_VARIABLES = {
     "now": "uint256",
     "this": "address",
@@ -31,7 +34,6 @@ SOLIDITY_VARIABLES_COMPOSED = {
     "tx.origin": "address",
 }
 
-
 SOLIDITY_FUNCTIONS: Dict[str, List[str]] = {
     "gasleft()": ["uint256"],
     "assert(bool)": [],
@@ -181,3 +183,34 @@ class SolidityFunction:
 
     def __hash__(self):
         return hash(self.name)
+
+
+class SolidityImportPlaceHolder(SolidityVariable):
+    """
+    Placeholder for import on top level objects
+    See the example at https://blog.soliditylang.org/2020/09/02/solidity-0.7.1-release-announcement/
+    In the long term we should remove this and better integrate import aliases
+    """
+
+    def __init__(self, import_directive: "Import"):
+        assert import_directive.alias is not None
+        super().__init__(import_directive.alias)
+        self._import_directive = import_directive
+
+    def _check_name(self, name: str):
+        return True
+
+    @property
+    def type(self) -> ElementaryType:
+        return ElementaryType("string")
+
+    def __eq__(self, other):
+        return (
+            self.__class__ == other.__class__
+            and self.name == other.name
+            and self._import_directive.filename == self._import_directive.filename
+        )
+
+    @property
+    def import_directive(self) -> "Import":
+        return self._import_directive
