@@ -8,10 +8,10 @@ class IncorrectPrinterInitialization(Exception):
 
 
 class AbstractPrinter(metaclass=abc.ABCMeta):
-    ARGUMENT = ''  # run the printer with slither.py --ARGUMENT
-    HELP = ''  # help information
+    ARGUMENT = ""  # run the printer with slither.py --ARGUMENT
+    HELP = ""  # help information
 
-    WIKI = ''
+    WIKI = ""
 
     def __init__(self, slither, logger):
         self.slither = slither
@@ -20,26 +20,31 @@ class AbstractPrinter(metaclass=abc.ABCMeta):
         self.logger = logger
 
         if not self.HELP:
-            raise IncorrectPrinterInitialization('HELP is not initialized {}'.format(self.__class__.__name__))
+            raise IncorrectPrinterInitialization(
+                "HELP is not initialized {}".format(self.__class__.__name__)
+            )
 
         if not self.ARGUMENT:
-            raise IncorrectPrinterInitialization('ARGUMENT is not initialized {}'.format(self.__class__.__name__))
+            raise IncorrectPrinterInitialization(
+                "ARGUMENT is not initialized {}".format(self.__class__.__name__)
+            )
 
         if not self.WIKI:
-            raise IncorrectPrinterInitialization('WIKI is not initialized {}'.format(self.__class__.__name__))
+            raise IncorrectPrinterInitialization(
+                "WIKI is not initialized {}".format(self.__class__.__name__)
+            )
 
     def info(self, info):
         if self.logger:
             self.logger.info(info)
 
-
     def generate_output(self, info, additional_fields=None):
         if additional_fields is None:
             additional_fields = {}
-        d = output.Output(info, additional_fields)
-        d.data['printer'] = self.ARGUMENT
+        printer_output = output.Output(info, additional_fields)
+        printer_output.data["printer"] = self.ARGUMENT
 
-        return d
+        return printer_output
 
     @abc.abstractmethod
     def output(self, filename):
