@@ -178,29 +178,31 @@ class PrinterCallGraph(AbstractPrinter):
             filename(string)
         """
 
+        all_contracts_filename = ""
         if not filename.endswith(".dot"):
-            filename += ".dot"
+            all_contracts_filename = f"{filename}.all_contracts.call-graph.dot"
         if filename == ".dot":
-            filename = "all_contracts.dot"
+            all_contracts_filename = "all_contracts.dot"
 
         info = ""
         results = []
-        with open(filename, "w", encoding="utf8") as f:
-            info += f"Call Graph: {filename}\n"
+        with open(all_contracts_filename, "w", encoding="utf8") as f:
+            info += f"Call Graph: {all_contracts_filename}\n"
             content = "\n".join(
                 ["strict digraph {"] + [_process_functions(self.slither.functions)] + ["}"]
             )
             f.write(content)
-            results.append((filename, content))
+            results.append((all_contracts_filename, content))
 
         for derived_contract in self.slither.contracts_derived:
-            with open(f"{derived_contract.name}.dot", "w", encoding="utf8") as f:
-                info += f"Call Graph: {derived_contract.name}.dot\n"
+            derived_output_filename = f"{filename}.{derived_contract.name}.call-graph.dot"
+            with open(derived_output_filename, "w", encoding="utf8") as f:
+                info += f"Call Graph: {derived_output_filename}\n"
                 content = "\n".join(
                     ["strict digraph {"] + [_process_functions(derived_contract.functions)] + ["}"]
                 )
                 f.write(content)
-                results.append((filename, content))
+                results.append((derived_output_filename, content))
 
         self.info(info)
         res = self.generate_output(info)
