@@ -5,13 +5,13 @@ from slither.slithir.utils.utils import is_valid_lvalue
 
 class Delete(OperationWithLValue):
     """
-        Delete has a lvalue, as it has for effect to change the value
-        of its operand
+    Delete has a lvalue, as it has for effect to change the value
+    of its operand
     """
 
     def __init__(self, lvalue, variable):
         assert is_valid_lvalue(variable)
-        super(Delete, self).__init__()
+        super().__init__()
         self._variable = variable
         self._lvalue = lvalue
 
