@@ -5,10 +5,11 @@ from slither.slithir.variables.reference import ReferenceVariable
 
 
 class Index(OperationWithLValue):
-
     def __init__(self, result, left_variable, right_variable, index_type):
-        super(Index, self).__init__()
-        assert is_valid_lvalue(left_variable) or left_variable == SolidityVariableComposed('msg.data')
+        super().__init__()
+        assert is_valid_lvalue(left_variable) or left_variable == SolidityVariableComposed(
+            "msg.data"
+        )
         assert is_valid_rvalue(right_variable)
         assert isinstance(result, ReferenceVariable)
         self._variables = [left_variable, right_variable]
@@ -36,4 +37,6 @@ class Index(OperationWithLValue):
         return self._type
 
     def __str__(self):
-        return "{}({}) -> {}[{}]".format(self.lvalue, self.lvalue.type, self.variable_left, self.variable_right)
+        return "{}({}) -> {}[{}]".format(
+            self.lvalue, self.lvalue.type, self.variable_left, self.variable_right
+        )
