@@ -4,9 +4,8 @@ from slither.core.solidity_types.type import Type
 
 
 class NewArray(Call, OperationWithLValue):
-
     def __init__(self, depth, array_type, lvalue):
-        super(NewArray, self).__init__()
+        super().__init__()
         assert isinstance(array_type, Type)
         self._depth = depth
         self._array_type = array_type
@@ -27,4 +26,6 @@ class NewArray(Call, OperationWithLValue):
 
     def __str__(self):
         args = [str(a) for a in self.arguments]
-        return '{} = new {}{}({})'.format(self.lvalue, self.array_type, '[]' * self.depth, ','.join(args))
+        return "{} = new {}{}({})".format(
+            self.lvalue, self.array_type, "[]" * self.depth, ",".join(args)
+        )
