@@ -1,14 +1,13 @@
-
-from .variable import SlithIRVariable
-from slither.core.variables.variable import Variable
 from slither.core.children.child_node import ChildNode
+from slither.core.variables.variable import Variable
+
 
 class TemporaryVariable(ChildNode, Variable):
 
     COUNTER = 0
 
     def __init__(self, node, index=None):
-        super(TemporaryVariable, self).__init__()
+        super().__init__()
         if index is None:
             self._index = TemporaryVariable.COUNTER
             TemporaryVariable.COUNTER += 1
@@ -16,7 +15,6 @@ class TemporaryVariable(ChildNode, Variable):
             self._index = index
         self._node = node
 
-
     @property
     def index(self):
         return self._index
@@ -27,8 +25,7 @@ class TemporaryVariable(ChildNode, Variable):
 
     @property
     def name(self):
-        return 'TMP_{}'.format(self.index)
+        return "TMP_{}".format(self.index)
 
     def __str__(self):
         return self.name
-
