@@ -1,14 +1,13 @@
-from .variable import SlithIRVariable
-from slither.core.variables.variable import Variable
 from slither.core.children.child_node import ChildNode
+from slither.slithir.variables.variable import SlithIRVariable
+
 
-from slither.core.solidity_types.type import Type
 class TupleVariable(ChildNode, SlithIRVariable):
 
     COUNTER = 0
 
     def __init__(self, node, index=None):
-        super(TupleVariable, self).__init__()
+        super().__init__()
         if index is None:
             self._index = TupleVariable.COUNTER
             TupleVariable.COUNTER += 1
@@ -27,7 +26,7 @@ class TupleVariable(ChildNode, SlithIRVariable):
 
     @property
     def name(self):
-        return 'TUPLE_{}'.format(self.index)
+        return "TUPLE_{}".format(self.index)
 
     def __str__(self):
         return self.name
