@@ -2,26 +2,23 @@ import logging
 import re
 from typing import List, TYPE_CHECKING, Union, Dict
 
+from slither.core.declarations.function_contract import FunctionContract
+from slither.core.expressions.literal import Literal
+from slither.core.solidity_types.array_type import ArrayType
 from slither.core.solidity_types.elementary_type import (
     ElementaryType,
     ElementaryTypeName,
 )
+from slither.core.solidity_types.function_type import FunctionType
+from slither.core.solidity_types.mapping_type import MappingType
 from slither.core.solidity_types.type import Type
 from slither.core.solidity_types.user_defined_type import UserDefinedType
-from slither.core.solidity_types.array_type import ArrayType
-from slither.core.solidity_types.mapping_type import MappingType
-from slither.core.solidity_types.function_type import FunctionType
-
 from slither.core.variables.function_type_variable import FunctionTypeVariable
-
-from slither.core.declarations.contract import Contract
-
-from slither.core.expressions.literal import Literal
-
 from slither.solc_parsing.exceptions import ParsingError
 
 if TYPE_CHECKING:
     from slither.core.declarations import Structure, Enum
+    from slither.core.declarations.contract import Contract
 
 logger = logging.getLogger("TypeParsing")
 
@@ -37,12 +34,14 @@ class UnknownType:  # pylint: disable=too-few-public-methods
         return self._name
 
 
-def _find_from_type_name(  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
+def _find_from_type_name(  # pylint: disable=too-many-locals,too-many-branches,too-many-statements,too-many-arguments
     name: str,
-    contract: Contract,
-    contracts: List[Contract],
-    structures: List["Structure"],
-    enums: List["Enum"],
+    functions_direct_access: List["Function"],
+    contracts_direct_access: List["Contract"],
+    structures_direct_access: List["Structure"],
+    all_structures: List["Structure"],
+    enums_direct_access: List["Enum"],
+    all_enums: List["Enum"],
 ) -> Type:
     name_elementary = name.split(" ")[0]
     if "[" in name_elementary:
@@ -60,19 +59,22 @@ def _find_from_type_name(  # pylint: disable=too-many-locals,too-many-branches,t
         name_contract = name_contract[len("contract ") :]
     if name_contract.startswith("library "):
         name_contract = name_contract[len("library ") :]
-    var_type = next((c for c in contracts if c.name == name_contract), None)
+    var_type = next((c for c in contracts_direct_access if c.name == name_contract), None)
 
     if not var_type:
-        var_type = next((st for st in structures if st.name == name), None)
+        var_type = next((st for st in structures_direct_access if st.name == name), None)
     if not var_type:
-        var_type = next((e for e in enums if e.name == name), None)
+        var_type = next((e for e in enums_direct_access if e.name == name), None)
     if not var_type:
         # any contract can refer to another contract's enum
         enum_name = name
         if enum_name.startswith("enum "):
             enum_name = enum_name[len("enum ") :]
-        all_enums = [c.enums for c in contracts]
-        all_enums = [item for sublist in all_enums for item in sublist]
+        elif enum_name.startswith("type(enum"):
+            enum_name = enum_name[len("type(enum ") : -1]
+        # all_enums = [c.enums for c in contracts]
+        # all_enums = [item for sublist in all_enums for item in sublist]
+        # all_enums += contract.slither.enums_top_level
         var_type = next((e for e in all_enums if e.name == enum_name), None)
         if not var_type:
             var_type = next((e for e in all_enums if e.canonical_name == enum_name), None)
@@ -82,8 +84,9 @@ def _find_from_type_name(  # pylint: disable=too-many-locals,too-many-branches,t
         if name_struct.startswith("struct "):
             name_struct = name_struct[len("struct ") :]
             name_struct = name_struct.split(" ")[0]  # remove stuff like storage pointer at the end
-        all_structures = [c.structures for c in contracts]
-        all_structures = [item for sublist in all_structures for item in sublist]
+        # all_structures = [c.structures for c in contracts]
+        # all_structures = [item for sublist in all_structures for item in sublist]
+        # all_structures += contract.slither.structures_top_level
         var_type = next((st for st in all_structures if st.name == name_struct), None)
         if not var_type:
             var_type = next((st for st in all_structures if st.canonical_name == name_struct), None)
@@ -98,11 +101,12 @@ def _find_from_type_name(  # pylint: disable=too-many-locals,too-many-branches,t
                 return ArrayType(UserDefinedType(var_type), Literal(depth, "uint256"))
 
     if not var_type:
-        var_type = next((f for f in contract.functions if f.name == name), None)
+        var_type = next((f for f in functions_direct_access if f.name == name), None)
     if not var_type:
         if name.startswith("function "):
             found = re.findall(
-                "function \(([ ()a-zA-Z0-9\.,]*?)\)(?: returns \(([a-zA-Z0-9() \.,]*)\))?", name
+                "function \(([ ()\[\]a-zA-Z0-9\.,]*?)\)(?: payable)?(?: (?:external|internal|pure|view))?(?: returns \(([a-zA-Z0-9() \.,]*)\))?",
+                name,
             )
             assert len(found) == 1
             params = [v for v in found[0][0].split(",") if v != ""]
@@ -110,10 +114,27 @@ def _find_from_type_name(  # pylint: disable=too-many-locals,too-many-branches,t
                 [v for v in found[0][1].split(",") if v != ""] if len(found[0]) > 1 else []
             )
             params = [
-                _find_from_type_name(p, contract, contracts, structures, enums) for p in params
+                _find_from_type_name(
+                    p,
+                    functions_direct_access,
+                    contracts_direct_access,
+                    structures_direct_access,
+                    all_structures,
+                    enums_direct_access,
+                    all_enums,
+                )
+                for p in params
             ]
             return_values = [
-                _find_from_type_name(r, contract, contracts, structures, enums)
+                _find_from_type_name(
+                    r,
+                    functions_direct_access,
+                    contracts_direct_access,
+                    structures_direct_access,
+                    all_structures,
+                    enums_direct_access,
+                    all_enums,
+                )
                 for r in return_values
             ]
             params_vars = []
@@ -131,7 +152,7 @@ def _find_from_type_name(  # pylint: disable=too-many-locals,too-many-branches,t
         if name.startswith("mapping("):
             # nested mapping declared with var
             if name.count("mapping(") == 1:
-                found = re.findall("mapping\(([a-zA-Z0-9\.]*) => ([a-zA-Z0-9\.\[\]]*)\)", name)
+                found = re.findall("mapping\(([a-zA-Z0-9\.]*) => ([ a-zA-Z0-9\.\[\]]*)\)", name)
             else:
                 found = re.findall(
                     "mapping\(([a-zA-Z0-9\.]*) => (mapping\([=> a-zA-Z0-9\.\[\]]*\))\)", name,
@@ -140,8 +161,24 @@ def _find_from_type_name(  # pylint: disable=too-many-locals,too-many-branches,t
             from_ = found[0][0]
             to_ = found[0][1]
 
-            from_type = _find_from_type_name(from_, contract, contracts, structures, enums)
-            to_type = _find_from_type_name(to_, contract, contracts, structures, enums)
+            from_type = _find_from_type_name(
+                from_,
+                functions_direct_access,
+                contracts_direct_access,
+                structures_direct_access,
+                all_structures,
+                enums_direct_access,
+                all_enums,
+            )
+            to_type = _find_from_type_name(
+                to_,
+                functions_direct_access,
+                contracts_direct_access,
+                structures_direct_access,
+                all_structures,
+                enums_direct_access,
+                all_enums,
+            )
 
             return MappingType(from_type, to_type)
 
@@ -158,29 +195,71 @@ def parse_type(t: Union[Dict, UnknownType], caller_context):
     from slither.solc_parsing.variables.function_type_variable import FunctionTypeVariableSolc
     from slither.solc_parsing.declarations.contract import ContractSolc
     from slither.solc_parsing.declarations.function import FunctionSolc
+    from slither.solc_parsing.slitherSolc import SlitherSolc
 
-    if isinstance(caller_context, ContractSolc):
-        contract = caller_context.underlying_contract
-        contract_parser = caller_context
-        is_compact_ast = caller_context.is_compact_ast
-    elif isinstance(caller_context, FunctionSolc):
-        contract = caller_context.underlying_function.contract
-        contract_parser = caller_context.contract_parser
-        is_compact_ast = caller_context.is_compact_ast
+    # Note: for convenicence top level functions use the same parser than function in contract
+    # but contract_parser is set to None
+    if isinstance(caller_context, SlitherSolc) or (
+        isinstance(caller_context, FunctionSolc) and caller_context.contract_parser is None
+    ):
+        if isinstance(caller_context, SlitherSolc):
+            sl = caller_context.core
+            next_context = caller_context
+        else:
+            assert isinstance(caller_context, FunctionSolc)
+            sl = caller_context.underlying_function.slither
+            next_context = caller_context.slither_parser
+        structures_direct_access = sl.structures_top_level
+        all_structuress = [c.structures for c in sl.contracts]
+        all_structures = [item for sublist in all_structuress for item in sublist]
+        all_structures += structures_direct_access
+        enums_direct_access = sl.enums_top_level
+        all_enumss = [c.enums for c in sl.contracts]
+        all_enums = [item for sublist in all_enumss for item in sublist]
+        all_enums += enums_direct_access
+        contracts = sl.contracts
+        functions = []
+    elif isinstance(caller_context, (ContractSolc, FunctionSolc)):
+        if isinstance(caller_context, FunctionSolc):
+            underlying_func = caller_context.underlying_function
+            # If contract_parser is set to None, then underlying_function is a functionContract
+            # See note above
+            assert isinstance(underlying_func, FunctionContract)
+            contract = underlying_func.contract
+            next_context = caller_context.contract_parser
+        else:
+            contract = caller_context.underlying_contract
+            next_context = caller_context
+
+        structures_direct_access = contract.structures + contract.slither.structures_top_level
+        all_structuress = [c.structures for c in contract.slither.contracts]
+        all_structures = [item for sublist in all_structuress for item in sublist]
+        all_structures += contract.slither.structures_top_level
+        enums_direct_access = contract.enums + contract.slither.enums_top_level
+        all_enumss = [c.enums for c in contract.slither.contracts]
+        all_enums = [item for sublist in all_enumss for item in sublist]
+        all_enums += contract.slither.enums_top_level
+        contracts = contract.slither.contracts
+        functions = contract.functions + contract.modifiers
     else:
         raise ParsingError(f"Incorrect caller context: {type(caller_context)}")
 
+    is_compact_ast = caller_context.is_compact_ast
     if is_compact_ast:
         key = "nodeType"
     else:
         key = "name"
 
-    structures = contract.structures + contract.slither.top_level_structures
-    enums = contract.enums + contract.slither.top_level_enums
-    contracts = contract.slither.contracts
-
     if isinstance(t, UnknownType):
-        return _find_from_type_name(t.name, contract, contracts, structures, enums)
+        return _find_from_type_name(
+            t.name,
+            functions,
+            contracts,
+            structures_direct_access,
+            all_structures,
+            enums_direct_access,
+            all_enums,
+        )
 
     if t[key] == "ElementaryTypeName":
         if is_compact_ast:
@@ -190,13 +269,25 @@ def parse_type(t: Union[Dict, UnknownType], caller_context):
     if t[key] == "UserDefinedTypeName":
         if is_compact_ast:
             return _find_from_type_name(
-                t["typeDescriptions"]["typeString"], contract, contracts, structures, enums,
+                t["typeDescriptions"]["typeString"],
+                functions,
+                contracts,
+                structures_direct_access,
+                all_structures,
+                enums_direct_access,
+                all_enums,
             )
 
         # Determine if we have a type node (otherwise we use the name node, as some older solc did not have 'type').
         type_name_key = "type" if "type" in t["attributes"] else key
         return _find_from_type_name(
-            t["attributes"][type_name_key], contract, contracts, structures, enums
+            t["attributes"][type_name_key],
+            functions,
+            contracts,
+            structures_direct_access,
+            all_structures,
+            enums_direct_access,
+            all_enums,
         )
 
     if t[key] == "ArrayTypeName":
@@ -204,25 +295,25 @@ def parse_type(t: Union[Dict, UnknownType], caller_context):
         if is_compact_ast:
             if t.get("length", None):
                 length = parse_expression(t["length"], caller_context)
-            array_type = parse_type(t["baseType"], contract_parser)
+            array_type = parse_type(t["baseType"], next_context)
         else:
             if len(t["children"]) == 2:
                 length = parse_expression(t["children"][1], caller_context)
             else:
                 assert len(t["children"]) == 1
-            array_type = parse_type(t["children"][0], contract_parser)
+            array_type = parse_type(t["children"][0], next_context)
         return ArrayType(array_type, length)
 
     if t[key] == "Mapping":
 
         if is_compact_ast:
-            mappingFrom = parse_type(t["keyType"], contract_parser)
-            mappingTo = parse_type(t["valueType"], contract_parser)
+            mappingFrom = parse_type(t["keyType"], next_context)
+            mappingTo = parse_type(t["valueType"], next_context)
         else:
             assert len(t["children"]) == 2
 
-            mappingFrom = parse_type(t["children"][0], contract_parser)
-            mappingTo = parse_type(t["children"][1], contract_parser)
+            mappingFrom = parse_type(t["children"][0], next_context)
+            mappingTo = parse_type(t["children"][1], next_context)
 
         return MappingType(mappingFrom, mappingTo)
 
